/* 
 * ==========================================================
 * Artekit demo sketch for the AK-DS1340 RTC breakout board
 * Copyright (c) 2014 Artekit Italy
 * http://www.artekit.eu
 *
 * Connect the pins in the following way:
 *
 * Arduino UNO          AK-DS1340
 * ==============================
 *   AN5                   SCL
 *   AN4                   SDA
 *   5V                    +5V
 *   GND                   GND
 *
 * This demo program shows a menu where you can configure
 * the AK-DS1340. Press any key to enter the menu.
 *
 * Product page:
 * http://www.artekit.eu/products/breakout-boards/ak-ds1340/
 *
 * ==========================================================
 */
 
#include <Wire.h>
#include <Artekit_DS1340.h>

Artekit_DS1340 ds1340;

/* Set this to one to access the menu mode */
static byte menu_mode = 0;

/* Reads and show the current time */
static void ShowCurrentTime()
{
  DATETIME dt;
  
  ds1340.GetTime(&dt);
  Serial.print("The current time is: ");
  
  /* Check if day_of_week has a good value */
  if (dt.day_of_week >= 1 && dt.day_of_week <= 7)
  {
    /* If so, print the string for the current day of the week */
    Serial.print(ds1340.GetDayOfWeek(dt.day_of_week));
    Serial.print(" ");
  }
  
  Serial.print(dt.month, DEC);
  Serial.print("/");
  Serial.print(dt.day, DEC);
  Serial.print("/");
  Serial.print(dt.year, DEC);
  Serial.print("  ");
  Serial.print(dt.hour, DEC);
  Serial.print(":");
  Serial.print(dt.minute, DEC);
  Serial.print(":");
  Serial.println(dt.second, DEC);
}

/* This function takes the user input from the serial and sets the time
 * to the DS1340
 */
static bool ReceiveTime(DATETIME* dt)
{
  int i;
  int val;
    
  Serial.println("Write the time in the following format: ");
  Serial.println("DMMDDYYHHMISS");
  Serial.println("D= Day of the week (1 to 7)");
  Serial.println("MM= Month");
  Serial.println("DD= Day");
  Serial.println("YY= Year");
  Serial.println("HH= Hour");
  Serial.println("MI= Minutes");
  Serial.println("SS= Seconds");
  Serial.println("For example: Monday 06/10/2013 19:01:22 will be 1061013190122");
  Serial.println("Press x to abort.");
  
  /* We know we have to read 13 values */
  for (i = 0; i < 13; i++)
  {
    while (!Serial.available());
    val = Serial.read();
    Serial.write(val);
    if (val == 'x' || val == 'X')
      return false;
 
    switch (i)
    {
      case 0:
        dt->day_of_week = (byte) (val - 48);
        break;
       
      case 1:
        dt->month = (byte) (val - 48) *10;
        break;
        
      case 2:
        dt->month += val - 48;
        break;
        
      case 3:
        dt->day = (byte) (val - 48) *10;
        break;
        
      case 4:
        dt->day += (byte) val - 48;
        break;
        
      case 5:
        dt->year = (byte) (val - 48) *10;
        break;
        
      case 6:
        dt->year += (byte) val - 48;
        dt->year += 2000;
        break;
       
      case 7:
        dt->hour = (byte) (val - 48) *10;
        break;
        
      case 8:
        dt->hour += (byte) val - 48;
        break;
        
      case 9:
        dt->minute = (byte) (val - 48) * 10;
        break;
        
      case 10:
        dt->minute += (byte) val - 48;
        break;
      
      case 11:
        dt->second = (byte) (val - 48) * 10;
        break;
        
      case 12:
        dt->second += (byte) val - 48;
        break;
    }
  }
    
  return true;
}

/* Ask the user the time and writes it to the DS1340 */
static void SetTime()
{
  DATETIME dt;
  
  if (ReceiveTime(&dt))
  {
    ds1340.SetTime(&dt);
    Serial.println("");
    ShowCurrentTime();
  }
}

/* Shows the menu */
static void ShowMenu()
{
 Serial.println("");
 Serial.println("Artekit AK-DS1340");
 Serial.println("-----------------");
 Serial.println("");
 Serial.println("Choose one of the following options:");
 Serial.println("S - Set time");
 Serial.println("R - Read time");
 Serial.println("E - Enable trickle charger");
 Serial.println("D - Disable trickle charger");
 Serial.println("T - Enable FT/OUT");
 Serial.println("O - Disable FT/OUT");
 Serial.println("X - Exit");
 Serial.println("");
}

void setup()
{
  Wire.begin();
  Serial.begin(9600);
}

void loop()
{
  /* We use the menu mode to configure the DS1340. If not in menu mode,
   * output the date every second
   */
  if (menu_mode)
  {
    switch (Serial.read())
    {
      case 's':
      case 'S':
        SetTime();
        ShowMenu();
        break;
        
      case 'r':
      case 'R':
        ShowCurrentTime();
        ShowMenu();
        break;
        
      case 'e':
      case 'E':
        ds1340.EnableTrickleCharger();
        Serial.println("Trickle charger enabled");
        ShowMenu();
        break;
        
      case 'd':
      case 'D':
        ds1340.DisableTrickleCharger();
        Serial.println("Trickle charger disabled");
        ShowMenu();
        break;
        
      case 't':
      case 'T':
        ds1340.EnableFTOUT();
        Serial.println("FT/OUT enabled");
        ShowMenu();
        break;
        
      case 'o':
      case 'O':
        ds1340.DisableFTOUT();
        Serial.println("FT/OUT disabled");
        ShowMenu();
        break;

      case 'x':
      case 'X':
        menu_mode = 0;
        break;
        
      case -1:
        break;
        
      default:
        Serial.println("Invalid option.");
        ShowMenu();
        break;
    }
  } else {
    if (Serial.available())
    {
      /* Enter menu mode when the user press a key */
      Serial.read();
      ShowMenu();
      menu_mode = 1;
    } else {
      /* Output current time every second */
      ShowCurrentTime();
      delay(1000);
    }
  }
}  
